require('./global/index');
const fs = require('fs');
const chalk = require('chalk');
// const webpack = require('webpack');
const path = require('path');
const qrcode = require('qrcode-terminal');
// const zipDir = require('zip-directory');
const superagent = require('superagent');
const extract = require('extract-zip');

const { launchStaticServer } = require('./pipeline/launch-server');
const { Compiler } = require('./pipeline/webpack-build');
const { zip } = require('./pipeline/zip');
const { getDebugUrl } = require('./pipeline/debug-tool');

const renderConfig = require('./webpack.render');
// const serviceConfig = require('./webpack.service');
const serviceConfigBase = require('./splitChunksConfig/webpack.service.base');
const serviceConfig = require('./splitChunksConfig/webpack.service');
const getIpAddress = require('./getIpAddress');

const ROOT = process.cwd();
const SRC = path.join(ROOT, 'src');
const DIST = path.join(ROOT, 'dist');
let wsInstance = null;

if (!fs.existsSync(path.join(ROOT, 'zip'))) {
    fs.mkdirSync(path.join(ROOT, 'zip'));
}

// zip 目录建立 static host，用于给 app 下载 zip 包
if (!process.env.TEST_BUILDER && process.argv.indexOf('no-serve') === -1) {
    launchStaticServer(path.join(ROOT, 'zip'));
}

// 强制校验 mid。
let midStr = '';
let showQR = true;
let ideMode = false;

process.argv.some(arg => {
    if (arg.indexOf('mid') === 0) {
        midStr = arg;
    }
    if (arg.indexOf('no-qr') === 0) {
        showQR = false;
    }
    if (arg.indexOf('no-serve') === 0) {
        showQR = false;
    }
    if (arg.indexOf('--ide') === 0) {
        ideMode = true;
        showQR = false;
    }
});

if (midStr === '') {
    if (!ideMode) {
        console.log('[biliSapp - error]:请使用 mid 命令行进行构建，命令：npm run build mid=xxx \r\n');
    }
    process.exit(1);
}
if (!ideMode) {
    console.log('[biliSapp - info] 校验 mid 成功！');
}

// const spinner = ora('building for pages...');
// spinner.start();

if (!fs.existsSync(path.join(ROOT, 'src/service.js'))) {
    fs.writeFileSync(path.join(ROOT, 'src/service.js'), '', 'utf-8');
}

let buildType = 'sapp';
let mode = 'build';
let base = null;
process.argv.forEach(arg => {
    if (arg.indexOf('--watch') === 0 || arg.indexOf('-w') === 0) {
        mode = 'watch';
    } else if (arg.indexOf('--widget') === 0) {
        buildType = 'widget';
    } else if (arg.indexOf('base') === 0) {
        base = arg.split('=')[1];
    }
});

if (!ideMode) {
    console.log('[biliSapp - info] 开始编译小程序...');
}

// const compiler = new Compiler([renderConfig(ROOT), serviceConfig(ROOT, SRC)]);
const compiler = new Compiler([renderConfig(ROOT, 'development'), serviceConfigBase(ROOT, SRC, 'development'), serviceConfig(ROOT, SRC, 'development')]);

if (mode === 'build') {
    compiler.build(false).then(() => {
        compilerDone();
    });
} else if (mode === 'watch') {
    compiler.watch(false, compilerDone);
}

async function compilerDone() {
    // fs.unlink(path.join(SRC, 'service.js'), () => {});
    if (!ideMode) {
        console.log('[biliSapp - info] 编译小程序完成');
    }

    // 兜底，分包如果没打出common.service.js，则手动写一个空的，因为客户端逻辑是加载到common.service.js才进行分包逻辑
    if (!fs.existsSync(path.join(DIST, './common.service.js'))) {
        fs.writeFileSync(path.join(DIST, './common.service.js'), '"";', 'utf-8');
    }

    // spinner.stop();

    const zipName = new Date().toJSON().replace(/[-:.TZ]/g, '') + '.zip';
    if (!fs.existsSync(path.join(ROOT, 'zip'))) {
        fs.mkdirSync(path.join(ROOT, 'zip'));
    }
    const zipPath = path.join(ROOT, 'zip', zipName);
    let zipLatestName = 'latest.zip';

    if (mode === 'watch') {
        zipLatestName = 'watch.zip';
    }
    const latestZipPath = path.join(ROOT, 'zip', zipLatestName);

    const zipFiles = [latestZipPath];
    if (mode === 'build') {
        zipFiles.push(zipPath);
    }

    if (base) {
        if (!path.isAbsolute(base)) {
            base = path.resolve(process.cwd(), base);
        }
        const targetDist = path.resolve(process.cwd(), 'dist/');

        if (!fs.existsSync(base)) {
            if (!ideMode) {
                console.error('[biliSapp - error]: base 路径不存在，请检查路径是否正确');
            }
            process.exit(1);
            return;
        }

        try {
            await extract(base, { dir: targetDist });
        } catch (e) {
            console.error(e);
            process.exit(1);
        }
    }

    zip(DIST, ...zipFiles).then(async() => {
        const ip = getIpAddress();
        const sapprc = JSON.parse(fs.readFileSync(path.join(ROOT, '.sapprc'), 'utf8'));
        let storageId = '';
        // let debugURL = `https://miniapp.bilibili.com/applet/debug/${sapprc.appId}_${sapprc.vAppId}?url=${encodeURIComponent(`http://${ip}:3000/${zipName}`)}`;
        // let debugURL = `https://miniapp.bilibili.com/applet/debug/${sapprc.appId}_${sapprc.vAppId}?url=${encodeURIComponent(`http://${ip}:3000/${zipLatestName}`)}`;
        let debugURL = getDebugUrl({
            type: buildType,
            ip,
            zipName: zipLatestName,
            sapprc,
            embedBase: !!base
        });

        if (midStr) {
            // mid=xxx
            let mid = midStr.slice(4);

            try {
                let uploadUrl = '';

                let upZipResponse = await superagent
                    .post('https://miniapp.bilibili.com/api/miniapp/file/upload')
                    .attach('file', latestZipPath, zipName)
                    .disableTLSCerts();
                if (upZipResponse.status === 200 && upZipResponse.body.code === 0 && upZipResponse.body.data) {
                    uploadUrl = upZipResponse.body.data.replace(/^http:\/\//, 'https://');
                } else {
                    if (!ideMode) {
                        console.log('[biliSapp - error]: 本地构建失败，请重试');
                    }
                    process.exit(1);
                    return;
                }

                // 本地调试校验参数判断
                let _tempParam = { appId: sapprc.appId, url: uploadUrl, isValidUrl: 0 };
                if (global._bili_cache_data.isInternal) {
                    _tempParam.staff = mid;
                } else {
                    _tempParam.mid = mid;
                }

                let upResponse = await superagent
                    .post('https://miniapp.bilibili.com/api/storage/dev_storage/upload')
                    .send(_tempParam)
                    .set('Accept', 'application/json')
                    .disableTLSCerts();
                if (upResponse.status === 200 && upResponse.body.code === 0) {
                    storageId = upResponse.body.data;
                    debugURL = getDebugUrl({
                        type: buildType,
                        ip,
                        zipName: zipLatestName,
                        sapprc,
                        storageId,
                        embedBase: !!base
                    });
                } else {
                    if (!ideMode) {
                        console.log('[biliSapp - error]: 本地构建失败，', upResponse.body.message);
                    }
                    process.exit(1);
                    return;
                }
            } catch (error) {
                if (!ideMode) {
                    console.log('[biliSapp - error]: 本地构建失败，请重试');
                }
                process.exit(1);
                return;
            }
        }

        if (process.env.TEST_BUILDER || !showQR) {
            if (ideMode) {
                console.log(chalk.cyan('  Build pages complete.\n'));
                console.log(chalk.cyan('  Build service complete.\n'));
            }
            process.exit(0);
            return;
        }

        let devtoolPort;
        // 调试工具静态端口一般从10022开始
        // 发现window版本扫描接口较慢，这里就不做过多的扫描了
        for (let i = 10022; i < 10028; i++) {
            try {
                const response = await superagent.post(`http://localhost:${i}/devtool-scan`).send({}).disableTLSCerts();
                if (response.status === 200 && response.text === 'small-app-devtool-application listened...') {
                    devtoolPort = i;
                    break;
                }
            } catch (error) {
                // console.error(error);
            }
        }
        if (devtoolPort) {
            await superagent.post(`http://localhost:${devtoolPort}/devtool-debug`).send({ debugURL }).disableTLSCerts();
            console.log(chalk.cyan('检测到调试工具正在运行，可以使用调试工具进行调试啦'));
        } else {
            console.log('[biliSapp - info] 生成二维码');
            qrcode.generate(debugURL, { small: true });
            console.log(debugURL);
            console.log(chalk.cyan('真机调试服务正在运行，可以用手机扫码调试构建出来的安装包啦'));
        }
        console.log(chalk.cyan('若要重新构建需退出（ctrl+c），再执行一次 npm run build'));

        wsInstance && wsInstance.send('hotreload');
    }).catch(e => {});
}
