const fs = require("fs");
const acorn = require("acorn");
const walk = require("acorn-walk");
const path = require("path");
const vueCompiler = require("@vue/component-compiler-utils");
const VueTemplateCompiler = require("vue-template-compiler");
const loaderUtils = require('loader-utils');
const escodegen = require("escodegen");
const extendsLoaderPath = path.resolve(
    __dirname,
    "../../../plugin/playlet-plugin/index_playlet.vue"
);
const { playerSlotDel } = require('./player-slot-del');

// https://mall.bilibili.com/small-app-online-build
module.exports = function blSappExtendsLoader(source) {

    const callback = this.async();
    const options = loaderUtils.getOptions(this);
    if(global.PLAYLET_PLUGIN_PLAYER_PATH&&global.PLAYLET_PLUGIN_PLAYER_PATH === this.resourcePath){
        return callback(null, playerSlotDel(source));
    }
    // 排除player 和 charge 组件被定义在页面文件夹下
    if (options.include && Array.isArray(options.include) && !options.include.some(prefix => this.resourcePath === path.join(prefix, 'index.vue'))) {
        return callback(null, source);
    }
    
    fs.readFile(extendsLoaderPath, "utf-8", (err, aContent) => {
        if (err) return callback(err);
        const aDescriptor = vueCompiler.parse({
            source: aContent,
            compiler: VueTemplateCompiler,
            needMap: false,
        });
        const bDescriptor = vueCompiler.parse({
            source: source,
            compiler: VueTemplateCompiler,
            needMap: false,
        });
        let bScriptContent = bDescriptor.script ? bDescriptor.script.content : "";
        // 排除引入 player 组件上方有注释 // 引入的情况，导致 player 模板内部被渲染
        const playerImportRegex = /^(?!\s*\/\/)\s*import\s+(\w*Player\w*)\s+from\s+['"]([^'"]+)['"]/im;

        const match = bScriptContent.match(playerImportRegex);
        if (match) {
            // const playerImportPath = match[1];
            const playerImportPath = match[2];
            const absolutePlayerImportPath = path.resolve(path.dirname(this.resourcePath), playerImportPath);
            global.PLAYLET_PLUGIN_PLAYER_PATH = absolutePlayerImportPath;
        }
        let templateContent = aDescriptor.template.content;
        let importMixinScript;
        // 替换 <slot name="Player"> 为 <Player>
        templateContent = templateContent.replace(/<slot\s+name="Player"/g, '<Player');
        // 替换 <slot name="Charge"> 为 <Charge>
        templateContent = templateContent.replace(/<slot\s+name="Charge"/g, '<Charge');

        // 替换 </slot> 为 </Player> 或 </Charge>，根据前面的标签
        templateContent = templateContent.replace(/<\/slot>/g, (match, offset, string) => {
            const precedingText = string.slice(0, offset);
            if (precedingText.includes('<Charge')) {
                return '</Charge>';
            } else if (precedingText.includes('<Player')) {
                return '</Player>';
            }
            return match;
        });
        const styleContent = aDescriptor.styles.map((style) => style.content).join("\n");
        const aScriptContent = aDescriptor.script ? aDescriptor.script.content : '';

        // 提取 methods 部分
        const extractMethods = (scriptContent) => {
            const ast = acorn.parse(scriptContent, { ecmaVersion: 2020, sourceType: "module" });
            let methods = '';
            walk.simple(ast, {
                Property(node) {
                    if (node.key.name === 'methods' && node.value.type === 'ObjectExpression') {
                        methods = scriptContent.slice(node.value.start + 1, node.value.end - 1);
                    }
                }
            });
            return methods;
        };
        const aMethods = extractMethods(aScriptContent);
        const bMethods = extractMethods(bScriptContent);

        // 合并 methods
        const mergedMethods = `${aMethods}${bMethods}`;

        if (aScriptContent && bScriptContent) {
            const pluginScriptContent = aScriptContent.replace('export default', 'const pluginMixin =');
            const mixinContent = `
                ${pluginScriptContent}
                export default pluginMixin;
            `;
            const mixinPath = path.resolve(__dirname, 'tempMixin.js');
            if (fs.existsSync(mixinPath)) {
                fs.unlinkSync(mixinPath);
            }
            fs.writeFileSync(mixinPath, mixinContent);
            importMixinScript = `
                import pluginMixin from '${mixinPath.replace(/\\/g, '\\\\')}';
            `;
        } else if (aScriptContent) {
            bScriptContent = aScriptContent;
        }
        // 移除原来的 methods 字段
        const ast = acorn.parse(bScriptContent, { ecmaVersion: 2020, sourceType: "module" });
        walk.ancestor(ast, {
            Property(node, state, ancestors) {
                if (node.key.name === 'methods' && node.value.type === 'ObjectExpression') {
                    const parent = ancestors[ancestors.length - 2];
                    if (parent && parent.type === 'ObjectExpression') {
                        parent.properties = parent.properties.filter(prop => prop !== node);
                    }
                }
            }
        });
        bScriptContent = escodegen.generate(ast);
        const scriptContent = `
            ${importMixinScript}
            ${bScriptContent.replace('export default {', `export default { \n  mixins: [pluginMixin], \n  methods: {${mergedMethods}},`)}
        `;
        
        const result = `
                <template>${templateContent}</template>
                <script>${scriptContent}</script>
                <style>${styleContent}</style>
            `;
        // console.log('result:', result);

        callback(null, result);

    });
};
