import { callNativeSync, callNative, typeCheck } from "./util";
import { compareVersion } from "./util/commonFn";
const emptyFn = () => {};

let rewardedVideoAdAdUnitIdMap = {
    single: {},
    multiton: {},
};


class RewardedVideoAd {
    constructor(adUnitId, multiton) {
        RewardedVideoAd.pageId = global.__biliCacheData.getCurrentPageId();
        RewardedVideoAd.adUnitId = adUnitId;
        this._destroyed = false;
        Object.defineProperties(this, {
            _id: {
                value: "",
                writable: true,
                enumerable: false,
                configurable: false,
            },
            _multiton: {
                value: multiton,
                writable: false,
                enumerable: false,
                configurable: false,
            },
            _closeListener: {
                value: [],
                writable: false,
                enumerable: false,
                configurable: false,
            },
            _errorListener: {
                value: [],
                writable: false,
                enumerable: false,
                configurable: false,
            },
            __frmErrorListener: {
                value: [],
                writable: false,
                enumerable: false,
                configurable: false,
            },
            _loadListener: {
                value: [],
                writable: false,
                enumerable: false,
                configurable: false,
            },
            __frmLoadListener: {
                value: [],
                writable: false,
                enumerable: false,
                configurable: false,
            },
        });
        const rewardedVideoSingle = rewardedVideoAdAdUnitIdMap.single[RewardedVideoAd.pageId];
        if (!multiton && rewardedVideoSingle) {
            return rewardedVideoSingle;
        } else {
            const { code, msg, data } = callNativeSync("createRewardedVideoAd", {
                adUnitId,
                multiton,
                pageId: RewardedVideoAd.pageId,
            });
            if (code != 0) {
                throw new Error(msg);
            }
            this._id = data.id;
            if (multiton) {
                
                if(rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId]){
                    rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId] = Object.assign(rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId],{[data.id]: this});
                }else{
                    rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId] = {
                        [data.id]: this,
                    };
                }
                // rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId][data.id] = this;
            } else {
                rewardedVideoAdAdUnitIdMap.single[RewardedVideoAd.pageId] = this;
            }
            // 创建广告时，主动调用一次 load
            this.load(2);
        }
    }
    load(type = 0) {
        const self = this;
        return new Promise((resolve, reject) => {
            // 兼容不传 adUnitId 报错的问题
            if (RewardedVideoAd.adUnitId === "0") {
                const errorObj = {
                    errCode: 1001,
                    errMsg: "参数错误",
                };
                return reject(errorObj);
            }
            function loaded() {
                self.offError(failed, { __framework: true });
                self.offLoad(loaded, { __framework: true });
                return resolve.apply(null, arguments);
            }

            function failed() {
                self.offError(failed, { __framework: true });
                self.offLoad(loaded, { __framework: true });
                return reject.apply(null, arguments);
            }
            self.onLoad(loaded, { __framework: true });
            const { code, msg } = callNativeSync("loadRewardedVideoAd", {
                id: this._id,
                framework: type
            });
            if (code != 0) {
                self.offLoad(loaded, { __framework: true });
                reject(msg);
            } else {
                self.onError(failed, { __framework: true });
            }
        });
    }
    show() {
        // 兼容不传 adUnitId 报错的问题
        if (RewardedVideoAd.adUnitId === "0") {
            const errorObj = {
                errCode: 1001,
                errMsg: "参数错误",
            };
            return Promise.reject(errorObj);
        }
        return new Promise((resolve, reject) => {
            callNative(
                "showRewardedVideoAd",
                Object.assign(
                    {},
                    { id: this._id },
                    {
                        success(res) {
                            resolve(res);
                        },
                        fail(e) {
                            reject(e);
                        },
                    }
                )
            );
        });
    }


    destroy() {
        const { code, msg } = callNativeSync("destroyRewardedVideoAd", {
            id: this._id,
        });
        this._destroyed = true;
        // 销毁之后，删除事件监听
        this._closeListener.length = 0;
        this._errorListener.length = 0;
        this.__frmErrorListener.length = 0;
        this._loadListener.length = 0;
        this.__frmLoadListener.length = 0;
        if (!this._multiton) {
            delete rewardedVideoAdAdUnitIdMap.single[RewardedVideoAd.pageId];
        } else {
            delete rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId][this._id];
        }
        if (code != 0) {
            throw new Error(msg);
        }
    }

    onClose(cb = emptyFn) {
        if (typeof cb !== "function") {
            throw new Error("参数必须是一个函数");
        }
        this._closeListener.push(cb);
    }

    offClose(cb) {
        if (typeof cb == "undefined") {
            this._closeListener.splice(0, this._closeListener.length);
            return;
        }

        const index = this._closeListener.indexOf(cb);
        if (index >= 0) {
            this._closeListener.splice(index, 1);
        }
    }

    onError(cb = emptyFn, opts = {}) {
        if (typeof cb !== "function") {
            throw new Error("参数必须是一个函数");
        }
        if (opts.__framework) {
            this.__frmErrorListener.push(cb);
        } else {
            this._errorListener.push(cb);
        }
    }

    offError(...args) {
        let [cb, opts = {}] = args;
        if (args.length == 1) {
            if (typeof args[0] == "object") {
                cb = undefined;
                opts = args[0];
            }
        }
        if (typeof cb == "undefined") {
            if (opts && opts.__framework) {
                this.__frmErrorListener.splice(0, this.__frmErrorListener.length);
            } else {
                this._errorListener.splice(0, this._errorListener.length);
            }
            return;
        }
        if (opts && opts.__framework) {
            let index = this.__frmErrorListener.indexOf(cb);
            if (index >= 0) {
                this.__frmErrorListener.splice(index, 1);
            }
        } else {
            let index = this._errorListener.indexOf(cb);
            if (index >= 0) {
                this._errorListener.splice(index, 1);
            }
        }
    }

    onLoad(cb = emptyFn, opts = {}) {
        if (typeof cb !== "function") {
            throw new Error("参数必须是一个函数");
        }
        if (opts.__framework) {
            this.__frmLoadListener.push(cb);
        } else {
            this._loadListener.push(cb);
        }
    }

    offLoad(...args) {
        let [cb, opts = {}] = args;
        if (args.length == 1) {
            if (typeof args[0] == "object") {
                cb = undefined;
                opts = args[0];
            }
        }
        if (typeof cb == "undefined") {
            if (opts && opts.__framework) {
                this.__frmLoadListener.splice(0, this.__frmLoadListener.length);
            } else {
                this._loadListener.splice(0, this._loadListener.length);
            }
            return;
        }
        if (opts && opts.__framework) {
            let index = this.__frmLoadListener.indexOf(cb);
            if (index >= 0) {
                this.__frmLoadListener.splice(index, 1);
            }
        } else {
            let index = this._loadListener.indexOf(cb);
            if (index >= 0) {
                this._loadListener.splice(index, 1);
            }
        }
    }
}
// adUnitId默认为 0，如果调用方不传adUnitId，调用 api 时报错
const createRewardedVideoAd = ({ adUnitId = "0", multiton = false } = {}) => {
    let result = typeCheck(adUnitId, "String");
    if (result != null) {
        throw new Error(
            `createRewardedVideoAd:fail parameter.adUnitId should be String instead of ${result};`
        );
    }
    const ad = new RewardedVideoAd(adUnitId, multiton);
    return ad;
};

let isHidden = false;
const rewardedVideoAdOnCloseCbs = [];
function wrapCb(cb) {
    const systemInfo = callNativeSync("getSystemInfoSync").data;
    if (systemInfo.platform === "ios") {
        return function realCb(...args) {
            if (!isHidden) {
                cb(...args);
            } else {
                rewardedVideoAdOnCloseCbs.push(() => {
                    cb(...args);
                });
            }
        };
    } else {
        return cb;
    }
}
function handleMessageAd(message, pageId) {
    const {type, event} = message;
    if(type === 'lifecycle'){
        switch (event) {
        case "onUnload":{
            delete rewardedVideoAdAdUnitIdMap.single[RewardedVideoAd.pageId];
            delete rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId];
            console.log('==============rewardedVideoAdAdUnitIdMap', rewardedVideoAdAdUnitIdMap);
            break;
        }
        }
        
    }else if(type === 'system'){
        const singleAd = pageId?rewardedVideoAdAdUnitIdMap.single[pageId]:rewardedVideoAdAdUnitIdMap.single[RewardedVideoAd.pageId];
        const multitonAd = pageId?rewardedVideoAdAdUnitIdMap.multiton[pageId]:rewardedVideoAdAdUnitIdMap.multiton[RewardedVideoAd.pageId];
        const ad = (singleAd && singleAd._id === message.id) ? singleAd : (multitonAd&&multitonAd[message.id]?multitonAd[message.id]:null);
        // const ad = singleAd?._id === message.id ? singleAd : multitonAd?.[message.id] ?? null;
        if (!ad) return;
        switch (event) {
        case "RewardedVideoAdOnClose": {
            const res = Object.assign(
                {},
                {
                    errMsg: "RewardedVideoAdOnClose",
                },
                message.data
            );
            if (res.isEnded === "false" || !res.isEnded) {
                res.isEnded = false;
            } else {
                res.isEnded = true;
            }
            const systemInfo = callNativeSync("getSystemInfoSync").data;

            // 兼容安卓栈顶获取有问题
            if(compareVersion(systemInfo.SDKVersion, '4.0.5') <= 0&&systemInfo.platform === "android"){
                setTimeout(() => {
                    ad._closeListener.forEach((cb) => {
                        wrapCb(cb)(res);
                    });
                },1);
            }else{
                ad._closeListener.forEach((cb) => {
                    wrapCb(cb)(res);
                });
            }
            // 兼容android 551,552  RewardedVideoAdOnClose 后漏调 RewardedVideoAdOnLoad 问题
            if (
                systemInfo.platform === "android" &&
            (systemInfo.SDKVersion === "2.8.0" || systemInfo.SDKVersion === "2.9.0")
            ) {
                const result = {
                    errMsg: "RewardedVideoAdOnLoad",
                };
                ad._loadListener.forEach((cb) => {
                    cb(result);
                });
            }
            ad.load(1);
            break;
        }
        case "RewardedVideoAdOnError": {
            const { errCode, errMsg } = message.data;
            const res = { errMsg, errCode };
            [...ad._errorListener, ...ad.__frmErrorListener].forEach((cb) => {
                cb(res);
            });
            break;
        }
        case "RewardedVideoAdOnLoad": {
            const res = {
                errMsg: "RewardedVideoAdOnLoad",
            };
            [...ad._loadListener, ...ad.__frmLoadListener].forEach((cb) => {
                cb(res);
            });
            break;
        }
        }
    }
   

}

export default { createRewardedVideoAd , handleMessageAd };
