/**
 * 开放能力
 */
import { callNative, emptyFn, callNativeSync, getUid } from './util';
import { compareVersion } from "./util/commonFn";

let globalLoadOPtions;

export default {
    /**
     * 调起用户编辑收货地址原生界面，并在编辑完成后返回用户选择的地址。
     *
     * @param {Object} options
     * @param {Function} options.success 成功回调
     * @param {Function} options.fail 失败回调
     */
    chooseAddress(options = {}) {
        callNative('chooseAddress', options);
    },
    /**
     * 获取用户授权
     *
     * @param {Object} options
     * @param {String} options.scope 授权的权限
     * @param {Function} options.success 成功回调
     * @param {Function} options.fail 失败回调
     */
    authorize(options = {}) {
        callNative('authorize', options, {scope: 'String'});
    },
    /**
     * 获取临时登录凭证（code）
     *
     * @param {Object} options
     * @param {Function} options.success 成功回调
     * @param {Function} options.fail 失败回调
     */
    login(options = {}) {
        callNative('login', options);
    },
    /**
     * 校验用户当前session_key是否有效。
     *
     * @param {Object} options
     * @param {Function} options.success 成功回调
     * @param {Function} options.fail 失败回调
     */
    checkSession(options = {}) {
        callNative('checkSession', options);
    },
    /**
     * 校验用户当前session_key是否有效
     *
     * @param {Object} options
     * @param {Boolean} options.withCredentials 是否带上登录态信息
     * @param {String} options.lang 显示用户信息的语言
     * @param {Function} options.success 成功回调
     * @param {Function} options.fail 失败回调
     */
    getUserInfo({
        withCredentials = false,
        lang = 'zh_CN',
        success = emptyFn,
        fail = emptyFn,
        complete = emptyFn
    } = {}) {
        const options = {
            withCredentials: !!withCredentials,
            lang,
            success,
            fail,
            complete
        };

        callNative('getUserInfo', options, {lang: 'String'});
    },
    /**
     * 发起支付请求
     *
     * @param {Object} options
     * @param {Function} options.success 成功回调
     * @param {Function} options.fail 失败回调
     */
    requestPayment(options = {}) {
        // 8.35.0 之后的版本支持
        const systemInfo = callNativeSync("getSystemInfoSync").data;        
        if(compareVersion(systemInfo.version, '8.35.0') < 0) {
            // 添加交换支付信息的接口调用
            bl.showLoading({title: '支付中,请稍后...'});
            const optionsLaunch = bl.getLaunchOptionsSync();
            // console.log('globalLoadOPtions', globalLoadOPtions);
            // console.log('小程序启动获取的参数：', optionsLaunch);
            const trace_info = Object.assign({}, globalLoadOPtions, optionsLaunch.query); 
            trace_info.router_episodeId = trace_info.bl_episode_id;
            trace_info.router_albumId = trace_info.bl_album_id;
            // console.log('传递给exchange_pay_info的启动参数：', trace_info);
            // const _report = function(key, type, params) {
            //     bl && bl.internal && bl.internal.reportNeuron({
            //         event: key,
            //         type,
            //         params
            //     });
            // };
            const _confirm = function(options, params) {
                // _report('miniapp.miniapp-window.pay.0.click', 2, { payParams: JSON.stringify(params) });
                bl.request({
                    url: 'https://miniapp.bilibili.com/open/open_api/v1/miniapp/payment/confirm_pay_info',
                    data: params,
                    method: 'POST',
                    success: function (__res) {
                        // console.log('exchange_confirm success', __res.data);
                        const { code } = __res.data;
                        if(code === 0) {
                            options.success && options.success(__res.data);  
                        } else {
                            options.fail && options.fail(__res.data);    
                        }
                    },
                    fail: function (__res) {
                        // console.log('exchange_confirm fail', __res);
                        options.fail && options.fail(__res);
                    },
                    complete: function (__res) {
                        bl.hideLoading();
                        // _report('miniapp.miniapp-window.pay.result.show', 3, __res.data);
                    }
                });
            };
            
            const exchangePayInfoData = {
                trace_info: trace_info,
                order_info: {
                    pay_info: options.pay_info,
                    order_id: options.order_id,
                },
            };
            // console.log('exchange_pay_info data', exchangePayInfoData);
            bl.request({
                url: `https://miniapp.bilibili.com/open/open_api/v1/miniapp/payment/exchange_pay_info`,
                data: exchangePayInfoData,
                method: 'POST',
                success: function (res) {
                    // console.log('exchange_pay_info success', res.data);
                    const { code, data } = res.data;
                    if(code === 0) {
                        const { pay_param, pay_type, pay_info} = data;
                        // sdk支付， 使用参数 pay_param
                        if(pay_type === 1){
                            // 开平支付， 使用参数 pay_info
                            bl.showModal({
                                title: pay_info.show_title,
                                content: pay_info.show_content,
                                success: function (_res) {
                                    if (_res.confirm) {
                                        _confirm(options, pay_info);
                                    } else if (_res.cancel) {
                                        options.fail && options.fail({errMsg: '用户点击取消'});
                                    }
                                }
                            });
                            
                        }else{
                            // 充值支付， 使用参数 pay_param 支付后，pay_info结算
                            callNative('requestPayment', Object.assign({}, pay_param, {
                                success: function (paymentRes) {
                                    options.success && options.success(paymentRes);
                                },
                                fail: function (paymentRes) {
                                    options.fail && options.fail(paymentRes);
                                }
                            }), {}, 'requestPayment', {
                                code: true
                            });
                        }
                    } else{
                        options.fail && options.fail(res.data);
                    }   
                },
                fail: function (res) {
                    // console.log('exchange_pay_info fail', res);
                },
                complete: function () {
                    bl.hideLoading();
                }
            });
        }else {
            callNative('requestPayment', options, {}, 'requestPayment', {
                code: true
            });
        }
    },
    // 合并 onLoad 传入的 options 参数给支付接口
    handleMessageRequestPayment(message, pageId){
        const {type, event} = message;
        
        if(type === 'lifecycle'){
            switch (event) {
            case "onLoad":{
                // 获取path后面的参数
                const path = message.path;
                const query = path.split('?')[1];
                const queryObj = {};
                if(query){
                    query.split('&').forEach(item => {
                        const [key, value] = item.split('=');
                        queryObj[key] = value;
                    });
                }
                globalLoadOPtions = queryObj;
                break;
            }
            }  
        }
    },
    /** 
     * @bldoc
     * 
     * @name getLaunchOptions2
     * @---@
     * 
     * @version 
     * 基础库 ios 1.1.1 安卓 1.1.2 开始支持
     * @---@
     * 
     * @desc 
     * 获取小程序启动时的参数。与 App.onLaunch 的回调参数一致。
     * @---@
     * 
     * @params
     * 属性 | 类型 | 默认值 | 必填 | 说明 | 版本
     * ---- | ---- | ---- | ---- | ---- | ----
     * success      | Function      |      | 否 | 收到开发者服务成功返回的回调函数 | 
     * fail         | Function      |      | 否 | 接口调用失败的回调函数 | 
     * @---@
     * 
     * @tips
     * ...
     */
    /**
     * 获取用户的当前设置。
     *
     * @param {Object} options
     * @param {Function} options.success 接口调用成功的回调函数
     * @param {Function} options.fail 接口调用失败的回调函数
     */
    getSetting(options = {}) {
        callNative('getSetting', options);
    },
    /**
     * 调起客户端小程序设置界面
     *
     * @param {Object} options
     * @param {Function} options.success 接口调用成功的回调函数
     * @param {Function} options.fail 接口调用失败的回调函数
     */
    openSetting(options = {}) {
        callNative('openSetting', options);
    },
    /**
     *  获取当前帐号信息
     *
     * @param {Object} [options={}]
     * @param {Function} options.success 接口调用成功的回调函数
     * @param {Function} options.fail 接口调用失败的回调函数
     */
    getAccountInfo(options = {}) {
        const callbackId = getUid();
        const successFn = typeof options.success === 'function' ? options.success : emptyFn;
        const failFn = typeof options.fail === 'function' ? options.fail : emptyFn;
        const completeFn = typeof options.complete === 'function' ? options.complete : emptyFn;

        const callbackFn = (result) => {
            if (result.code === 0) {
                if (!result.data.miniProgram) {
                    result.data = {
                        miniProgram: result.data
                    };
                }

                successFn(Object.assign({ errMsg: 'getAccountInfo:ok' }, result.data));
                completeFn(Object.assign({ errMsg: 'getAccountInfo:ok' }, result.data));
            } else {
                failFn({
                    errMsg: result.msg
                });
                completeFn({
                    errMsg: result.msg
                });
            }
        };

        GLOBAL_OBJECT.callbacks[callbackId] = callbackFn;
        GLOBAL_OBJECT.callNative('getAccountInfo', JSON.stringify(options), callbackId);
    },
    /**
     *  获取当前帐号信息
     */
    getAccountInfoSync() {
        let result = callNativeSync('getAccountInfoSync').data;

        if (!result.miniProgram) {
            result = {
                miniProgram: result
            };
        }

        return result;
    },
    /**
     * 支持AR
     */
    createFaceDetection({
        modelSelection = 0,
        success = emptyFn,
        fail = emptyFn,
        complete = emptyFn
    } = {}) {
        const options = {
            modelSelection,
            success,
            fail,
            complete
        };
        const _sys = bl.getSystemInfoSync();
        // 仅安卓
        if (_sys.platform.toLowerCase() === 'android' && global.createFaceDetection) {
            global.createFaceDetection(options);
        } else {
            options.fail && options.fail({ errMsg: 'not support' });
            options.complete && options.complete({ errMsg: 'not support' });
        }
    }
};
